# -*- coding: utf-8 -*-
"""
Seismic phase file parser

@author: lezhao.jia@gmail.com

2025.09.20 update
"""
import pandas as pd
from io import StringIO
import time,datetime
import os

def Dis_seismic_phaseRTS(path):
    df_phase = pd.read_csv(
        path,
        sep='\t',
        encoding='utf-8',
        parse_dates=False
        )
    return df_phase

def Dis_seismic_phase(path):
    '''
    Parse seismic phase file

    Parameters
    ----------
    path : str
        Path to phase file, supports Jopens format or simple tab-separated format

    Returns
    -------
    DataFrame
        Phase arrival data in DataFrame format

    '''
    path=os.path.abspath(path)
    with open(path,'r',encoding='utf-8') as f:
      read_data=f.read()
      f.closed
    
    try:
        Seis_info=read_data.split('#Phase Arrivals:')  
        Phase_info=Seis_info[1].split('#Station Magnitudes:')
        Seis_info=Seis_info[0]
        Mag_info=Phase_info[1]
        Phase_info=Phase_info[0]
        
        df_phase=pd.read_csv(StringIO(Phase_info), delim_whitespace=True,header=0,names=["id", "dist", "azi", "phase","date","time","res","wt"],index_col=0)
        df_mag=pd.read_csv(StringIO(Mag_info), delim_whitespace=True,header=0,names=["id", "dist", "azi", "type","date","time","value","res","amp","per"]) 
        
        Seis_info=Seis_info.split(sep=None)
        O_time=datetime.datetime(int(Seis_info[0]),int(Seis_info[1]),int(Seis_info[2]),int(Seis_info[3]),int(Seis_info[4]),int(float(Seis_info[5])))
        lat=Seis_info[6]
        lon=Seis_info[8]
        dep=Seis_info[10]
        mag=Seis_info[12]
        mag_flag=Seis_info[13]
        
        df_phase['O_time']=O_time
        df_phase['lat']=lat
        df_phase['lon']=lon
        df_phase['dep']=dep
        df_phase['mag']=mag
        df_phase['mag_flag']=mag_flag
    except (IndexError, ValueError):
        df_phase=pd.read_csv(path, sep='\t', encoding='utf-8', header=0)
        if 'Phase_name' in df_phase.columns:
            df_phase=df_phase.rename(columns={'Phase_name':'phase', 'Phase_time':'date', 'Phase_time_frac':'time', 'Distance':'dist', 'Azi':'azi'})
        if 'id' not in df_phase.columns:
            if all(col in df_phase.columns for col in ['Net_code', 'Sta_code', 'Loc_id', 'Chn_code']):
                df_phase['Net_code']=df_phase['Net_code'].astype(str)
                df_phase['Sta_code']=df_phase['Sta_code'].astype(str)
                df_phase['Loc_id']=df_phase['Loc_id'].astype(str)
                df_phase['Chn_code']=df_phase['Chn_code'].astype(str)
                df_phase['id']=df_phase['Net_code']+'.'+df_phase['Sta_code']+'.'+df_phase['Loc_id']+'.'+df_phase['Chn_code']
            else:
                df_phase['id']=[str(i) for i in range(len(df_phase))]
        if 'O_time' not in df_phase.columns:
            df_phase['O_time']=datetime.datetime.now()
        if 'lat' not in df_phase.columns:
            df_phase['lat']=0.0
        if 'lon' not in df_phase.columns:
            df_phase['lon']=0.0
        if 'dep' not in df_phase.columns:
            df_phase['dep']=0.0
        if 'mag' not in df_phase.columns:
            df_phase['mag']=0.0
        if 'mag_flag' not in df_phase.columns:
            df_phase['mag_flag']=''
    
    return df_phase





# -*- coding: utf-8 -*-
"""
seisclass - Seismic waveform classification package

This package provides functions to classify seismic waveforms using pre-trained models.
"""

__version__ = "0.1.1"

from .check_wave import check_seed

__all__ = ["check_seed"]



# -*- coding: utf-8 -*-
"""
Seismic waveform classification module
@author: jialuozhao

"""

import pandas as pd
import os,sys
from datetime import datetime,timedelta 
import numpy as np 
import obspy
from obspy.core import UTCDateTime

import joblib
import warnings

os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2'

package_dir = os.path.dirname(os.path.abspath(__file__))

try:
    from .packages import read_phase
except ImportError:
    import os
    import sys
    sys.path.insert(0, os.path.dirname(os.path.abspath(__file__)))
    from packages import read_phase

def return_result(x):
    x=np.argmax(x)
    if(x==0):
        return 'Earthquake'
    else:
        return 'Non-earthquake'

def check_seed(seed_path, phase_path, model_str='251111nw'):
    '''
    Classify seismic event from SEED file

    Parameters
    ----------
    seed_path : str
        Path to SEED file
    phase_path : str
        Path to phase file
    model_str : str, optional
        Model name, default is '251111nw'

    Returns
    -------
    str
        Comma-separated result string: event_type,earthquake_prob,explode_prob,collapse_prob

    '''

    model_name = model_str
    model_file = os.path.join(package_dir, 'model', model_name, 'event_model.json')
    model_weight = os.path.join(package_dir, 'model', model_name, 'event_model.h5')
    model_pkl = os.path.join(package_dir, 'model', model_name, 'event_model.pkl')

    file_path = seed_path
    phase_path = phase_path

    (Event_id, Event_type) = os.path.splitext(os.path.basename(file_path))

    with open(model_file, 'r') as file:
        model_json = file.read()

    import tensorflow as tf

    tf.config.optimizer.set_jit(True)
    with tf.device('/CPU:0'):
        new_model = tf.keras.models.model_from_json(model_json)
        new_model.load_weights(model_weight)

    wave_array = np.empty(0)
    pic_result = ''
    
    st = obspy.read(file_path)
    
    df_phase = read_phase.Dis_seismic_phase(phase_path)
    df_p_phase = df_phase[df_phase['phase'].str.startswith('P')]
    
    df_temp = df_p_phase['id'].str.split('.', expand=True)
    df_p_phase.loc[:, 'Net_code'] = df_temp[0]
    df_p_phase.loc[:, 'Sta_code'] = df_temp[1]
    
    if df_p_phase['time'].dtype != 'object':
        df_p_phase['time'] = df_p_phase['time'].astype(str)
    if df_p_phase['date'].dtype != 'object':
        df_p_phase['date'] = df_p_phase['date'].astype(str)
    
    def format_time(row):
        date_str = row['date']
        time_str = row['time']
        combined_str = f"{date_str} {time_str}"
        if ' ' in combined_str:
            parts = combined_str.split(' ')
            if len(parts) >= 3:
                date_part = parts[0]
                time_part = parts[1]
                ms_part = parts[2]
                ms_part = ms_part.ljust(4, '0')[:4]
                return f"{date_part} {time_part}.{ms_part}"
        return combined_str
    
    df_p_phase['Phase_mtime'] = df_p_phase.apply(format_time, axis=1)
    df_p_phase['Phase_mtime'] = pd.to_datetime(df_p_phase['Phase_mtime'], errors='coerce')
    
    i = 0
    for row in df_p_phase.itertuples():
        phase_mtime = getattr(row, 'Phase_mtime')
        if pd.isna(phase_mtime):
            continue
        rowst = st.select(network=getattr(row, 'Net_code'), station=getattr(row, 'Sta_code'))
        start = UTCDateTime(phase_mtime - timedelta(seconds=3) - timedelta(hours=8))
        
        j = 0
        channel_count = 0
        for j in range(0, rowst.count()):
            if channel_count == 3:
                break
            if rowst[j].stats.endtime < (start + 60):
                continue
            x = rowst[j].slice(starttime=start, endtime=start + 60).data
            if len(x) != 6001:
                continue
            wave_array = np.append(wave_array, x)
            channel_count += 1
        i += 1

    if wave_array.shape[0] == 0:
        return ('Earthquake,0,0,0', pic_result)

    wave_array = wave_array.reshape(-1, 18003)
    wave_array = wave_array[:, 0:18000]
    
    col = np.around(np.mean(wave_array, axis=1).reshape(-1, 1))
    ins = np.where(wave_array == 0)
    wave_array[ins] = np.take(col, ins[0])

    try:
        scaler = joblib.load(model_pkl)
        wave_array = scaler.transform(wave_array)
    except Exception as e:
        warnings.warn("Scaler load failed. Using fallback normalization.", UserWarning)
        wave_array = (wave_array - np.mean(wave_array)) / (np.max(wave_array) - np.min(wave_array) + 1e-8)
    
    wave_array = wave_array.reshape(-1, 100, 180, 1)
        
    np.set_printoptions(suppress=True)

    predictions = new_model.predict(wave_array)

    eathquake = round((np.sum(predictions[:, 0]) / predictions.shape[0]) * 100, 2)
    explode = round((np.sum(predictions[:, 1]) / predictions.shape[0]) * 100, 2)
    collapse = round((np.sum(predictions[:, 2]) / predictions.shape[0]) * 100, 2)
    
    return '%s,%s,%s,%s' % (return_result((eathquake, explode, collapse)), eathquake, explode, collapse)




import os
from seisclass import check_seed


def test_check_seed():
    """Test check_seed function"""
    # Get test data path
    test_data_dir = os.path.join(os.path.dirname(__file__), '..', 'seisclass', 'resource')
    test_seed = os.path.join(test_data_dir, 'test.seed')
    test_phase = os.path.join(test_data_dir, 'test.phase')
    
    # Simulate actual user usage
    print("\n=== Testing seisclass library usage ===")
    print(f"SEED file used: {test_seed}")
    print(f"Phase file used: {test_phase}")
    
    try:
        # Call function (consistent with user code)
        result = check_seed(test_seed, test_phase)
        
        # Output result
        print(f"\n=== Test result ===")
        print(f"Classification result: {result}")
        print("Test successful!")
        
    except Exception as e:
        print(f"\nTest failed: {e}")


if __name__ == "__main__":
    """Simulate user usage scenario when running directly"""
    test_check_seed()


from setuptools import setup, find_packages

setup(
    name="seisclass",
    version="0.1.1",
    author="Jia Luozhao",
    author_email="18429320@qq.com",
    description="Seismic event classification package",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/yourusername/seisclass",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        "numpy>=1.19.0",
        "obspy>=1.2.0",
        "tensorflow>=2.0.0",
        "keras>=2.3.0",
        "joblib>=1.0.0",
        "scikit-learn>=0.24.0",
        "pandas>=1.1.0"
    ],
    package_data={
        'seisclass': ['model/251111nw/*', 'resource/*'],
    },
)
